# Copyright (c) 2015. Mount Sinai School of Medicine
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Helpers to convert between data representations
"""

from __future__ import print_function, division, absolute_import
from collections import defaultdict

import pandas as pd

def extract_mutant_peptides(effects, padding_around_mutation):
    """
    Map each effect onto a source sequence, and start/end of an interval
    from which the caller can construct a peptide subsequence containing
    mutated residues
    """
    result = {}
    for effect in effects:
        seq = effect.mutant_protein_sequence
        # some effects will lack a mutant protein sequence since
        # they are either silent or unpredictable
        if seq:
            mutation_start = effect.aa_mutation_start_offset
            mutation_end = effect.aa_mutation_end_offset
            start = max(0, mutation_start - padding_around_mutation)
            end = min(len(seq), mutation_end + padding_around_mutation)
            result[effect] = (seq, start, end)
    return result

def contains_mutation(binding_prediction, effect):
    peptide_start = binding_prediction.offset
    peptide_end = binding_prediction.offset + binding_prediction.length - 1
    return (
        peptide_start < effect.aa_mutation_end_offset and
        peptide_end >= effect.aa_mutation_start_offset
    )

def epitopes_to_dataframe(
        epitope_collection,
        gene_expression_dict=None,
        transcript_expression_dict=None):
    """
    An mhctools.EpitopeCollection creates a very noisy DataFrame representation
    (since it doesn't know e.g. that Topiary's sequence keys are variants).
    So, here's some specialized logic for making a DataFrame from a
    Variant-specific EpitopeCollection.

    Optional arguments for expression level dictionaries will cause the
    data frame to have columns 'gene_expression' and/or 'transcript_expression'.
    """
    column_dict = defaultdict(list)

    # list of column names and functions to extract the value for that column
    # from a single binding prediction.
    # Important for understanding this code: each element of epitope_collection
    # is an mhctools.BindingPrediction object whose `source_sequence_key` field
    # is being abused to store a Varcode.MutationEffect object (which acts as
    # a unique key for the longer amino acid sequences out of which we pulled
    # candidate epitopes)

    def get_effect(binding_prediction):
        return binding_prediction.source_sequence_key

    def get_variant(binding_prediction):
        return get_effect(binding_prediction).variant

    simple_column_extractors = [
        ("allele", lambda x: x.allele),
        ("peptide", lambda x: str(x.peptide)),
        ("length", lambda x: x.length),
        ("ic50", lambda x: x.value),
        ("percentile_rank", lambda x: x.percentile_rank),
        ("gene", lambda x: get_effect(x).gene_name),
        ("transcript", lambda x: get_effect(x).transcript_name),
        ("variant", lambda x: get_variant(x).short_description),
        ("effect", lambda x: get_effect(x).short_description),
        ("effect_type", lambda x: get_effect(x).__class__.__name__),
        ("prediction_method", lambda x: x.prediction_method_name),
        ("protein_sequence",
            lambda x: str(get_effect(x).mutant_protein_sequence)),
        ("protein_mutation_start",
            lambda x: get_effect(x).aa_mutation_start_offset),
        ("protein_mutation_end",
            lambda x: get_effect(x).aa_mutation_end_offset),
        ("peptide_offset_in_protein", lambda x: x.offset),
    ]
    if gene_expression_dict:
        key_fn_pair = (
            "gene_expression",
            lambda x: gene_expression_dict.get(get_effect(x).gene_id, 0.0)
        )
        simple_column_extractors.append(key_fn_pair)

    if transcript_expression_dict:
        key_fn_pair = (
            "transcript_expression",
            lambda x: transcript_expression_dict.get(
                get_effect(x).transcript_id, 0.0)
        )
        simple_column_extractors.append(key_fn_pair)

    for x in epitope_collection:
        for column_name, fn in simple_column_extractors:
            column_dict[column_name].append(fn(x))
        effect = x.source_sequence_key

        # half-open interval of mutated residues in the source sequence
        # the peptide was extracted from
        mut_start_in_source_seq = effect.aa_mutation_start_offset
        mut_end_in_source_seq = effect.aa_mutation_end_offset
        # TODO: write unit tests for all interval logic,
        # since it's easy to get wrong
        if contains_mutation(x, effect):
            # need a half-open start/end interval
            peptide_mut_start = min(
                x.length,
                max(0, mut_start_in_source_seq - x.offset))
            peptide_mut_end = min(
                x.length,
                max(0, mut_end_in_source_seq - x.offset))
            mutated = True
        else:
            peptide_mut_start = peptide_mut_end = None
            mutated = False
        column_dict['peptide_mutation_start'].append(peptide_mut_start)
        column_dict['peptide_mutation_end'].append(peptide_mut_end)
        column_dict['peptide_contains_mutation'].append(mutated)
    column_names = [pair[0] for pair in simple_column_extractors] + [
        'peptide_mutation_start',
        'peptide_mutation_end',
        'peptide_contains_mutation']
    return pd.DataFrame(
        column_dict,
        columns=column_names)

def epitopes_to_csv(epitope_collection, csv_path):
    df = epitopes_to_dataframe(epitope_collection)
    df.to_csv(csv_path, index=False)



from .args import (
    arg_parser,
    variant_collection_from_args,
    mhc_binding_predictor_from_args
)
from .mutant_epitope_predictor import MutantEpitopePredictor
from .epitope_helpers import (
    epitopes_to_dataframe,
    epitopes_to_csv
)

__all__ = [
    "MutantEpitopePredictor",
    "arg_parser",
    "variant_collection_from_args",
    "mhc_binding_predictor_from_args",
    "epitopes_to_dataframe",
    "epitopes_to_csv",
]


# Copyright (c) 2015. Mount Sinai School of Medicine
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function, division, absolute_import
import logging

from typechecks import require_integer
from varcode import NonsilentCodingMutation
from mhctools import EpitopeCollection, BindingPrediction

from .epitope_helpers import extract_mutant_peptides, contains_mutation

def _apply_filter(
        filter_fn,
        collection,
        result_fn=None,
        filter_name="",
        collection_name=""):
    """
    Apply filter to effect collection and print number of dropped elements
    """
    n_before = len(collection)
    filtered = [x for x in collection if filter_fn(x)]
    n_after = len(filtered)
    if not collection_name:
        collection_name = collection.__class__.__name__
    logging.info(
        "%s filtering removed %d/%d entries of %s",
        filter_name,
        (n_before - n_after),
        n_before,
        collection_name)
    if result_fn:
        return result_fn(filtered)
    else:
        return filtered

def _dict_from_namedtuple(obj):
    return {
        field_name: getattr(obj, field_name)
        for field_name in obj._fields
    }

class MutantEpitopePredictor(object):
    def __init__(
            self,
            mhc_model,
            padding_around_mutation=0,
            ic50_cutoff=None,
            percentile_cutoff=None,
            keep_wildtype_epitopes=False):
        """
        Parameters
        ----------
        mhc_model : object
            MHC Class I binding prediction model which expected to have a
            property 'epitope_lengths' that is a list of strings and
            method called 'predict' that takes a dictionary of sequences and
            returns an mhctools.EpitopeCollection.

        padding_around_mutation : int, optional
            Number of amino acids on each side of the mutation to include
            in candidate epitopes. If not given, then padding_around_mutation
            is set to one less than the maximum epitope length.

        ic50_cutoff : float, optional
            Drop any binding predictions with IC50 nM weaker than this cutoff.

        percentile_cutoff : float, optional
            Drop any binding predictions with percentile rank below this cutoff.

        keep_wildtype_epitopes : bool
            Keep epitopes which don't contain mutated residues
        """
        require_integer(
            padding_around_mutation, "Padding around mutated residues")

        self.mhc_model = mhc_model

        # dictionary mapping tuples of HLA alleles to instances
        self.mhc_model_instances = {}

        self.epitope_lengths = mhc_model.epitope_lengths
        if min(self.epitope_lengths) < 1:
            raise ValueError(
                "Epitope lengths must be positive integers, got: %s" % (
                    min(self.epitope_lengths),))
        # even if user doesn't want any padding around the mutation we need
        # to at least include enough of the surrounding non-mutated
        # residues to construct candidate epitopes of the specified lengths
        min_required_padding = max(self.epitope_lengths) - 1
        padding_around_mutation = max(
            padding_around_mutation, min_required_padding)
        self.padding_around_mutation = padding_around_mutation
        self.ic50_cutoff = ic50_cutoff
        self.percentile_cutoff = percentile_cutoff
        self.keep_wildtype_epitopes = keep_wildtype_epitopes

    def epitopes_from_mutation_effects(
            self,
            effects,
            gene_expression_dict=None,
            gene_expression_threshold=0.0,
            transcript_expression_dict=None,
            transcript_expression_threshold=0.0):
        """Given a Varcode.EffectCollection of predicted protein effects,
        return a DataFrame of the predicted epitopes around each
        mutation.

        Parameters
        ----------
        effects : Varcode.EffectCollection

        mhc_alleles : list of strings
            MHC alleles

        gene_expression_dict : dict
            Dictionary mapping gene IDs to RNA expression estimates

        gene_expression_threshold : float
            If gene_expression_dict is given, only keep effects on genes
            expressed above this threshold.

        transcript_expression_dict : dict
            Dictionary mapping transcript IDs to RNA expression estimates

        transcript_expression_threshold : float
            If transcript_expression_dict is given, only keep effects on
            transcripts above this threshold.
        """
        # we only care about effects which impac the coding sequence of a
        # protein
        effects = _apply_filter(
            lambda effect: isinstance(effect, NonsilentCodingMutation),
            effects,
            result_fn=effects.clone_with_new_elements,
            filter_name="Silent mutation")
        if gene_expression_dict:
            effects = _apply_filter(
                lambda effect: (
                    gene_expression_dict.get(effect.gene_id, 0.0) >
                    gene_expression_threshold),
                effects,
                result_fn=effects.clone_with_new_elements,
                filter_name="Gene expression")
        if transcript_expression_dict:
            _apply_filter(
                lambda effect: (
                    transcript_expression_dict.get(effect.transcript_id, 0.0) >
                    transcript_expression_threshold
                ),
                effects,
                result_fn=effects.clone_with_new_elements,
                filter_name="Transcript expression")

        variant_effect_groups = effects.groupby_variant()

        if len(variant_effect_groups) == 0:
            logging.warn("No candidates for MHC binding prediction")
            return EpitopeCollection([])

        if transcript_expression_dict:
            # if expression data is available, then for each variant
            # keep the effect annotation for the most abundant variant
            def reduce_variant_effect_group(variant_effects):
                return variant_effects.top_expression_effect(
                    transcript_expression_dict)
        else:
            # if no transcript abundance data is available, then
            # for each variant keep the effect with the most significant
            # predicted effect on the protein sequence, along with using
            # transcript/CDS length as a tie-breaker for effects with the same
            # priority.
            def reduce_variant_effect_group(variant_effects):
                return variant_effects.top_priority_effect()

        final_effects = [
            reduce_variant_effect_group(variant_effects)
            for variant_effects in variant_effect_groups.values()
        ]

        peptide_interval_dict = extract_mutant_peptides(
            effects=final_effects,
            padding_around_mutation=self.padding_around_mutation)
        peptide_dict = {
            effect: source_sequence[start:end]
            for (effect, (source_sequence, start, end))
            in peptide_interval_dict.items()
        }
        # adjust offsets and source sequences of peptides in binding
        # predictions to reflect the longer source sequence they come from
        binding_predictions = []
        for x in self.mhc_model.predict(peptide_dict):
            effect = x.source_sequence_key
            (source_sequence, source_start, _) = peptide_interval_dict[effect]
            fields = _dict_from_namedtuple(x)
            fields["source_sequence"] = source_sequence
            fields["offset"] = fields["offset"] + source_start
            binding_predictions.append(BindingPrediction(**fields))

        # filter out low binders
        if self.ic50_cutoff:
            binding_predictions = _apply_filter(
                filter_fn=lambda x: x.value <= self.ic50_cutoff,
                collection=binding_predictions,
                filter_name="IC50 nM cutoff",
                collection_name="binding predictions",
            )
        if self.percentile_cutoff:
            binding_predictions = _apply_filter(
                filter_fn=lambda x: x.percentile_rank <= self.percentile_cutoff,
                collection=binding_predictions,
                filter_name="IC50 percentile rank cutoff",
                collection_name="binding predictions",
            )

        if not self.keep_wildtype_epitopes:
            binding_predictions = _apply_filter(
                filter_fn=lambda x: contains_mutation(x, x.source_sequence_key),
                collection=binding_predictions,
                filter_name="Wildtype epitope",
                collection_name="binding predictions",
            )
        return EpitopeCollection(binding_predictions)

    def epitopes_from_variants(
            self,
            variants,
            gene_expression_dict=None,
            gene_expression_threshold=0.0,
            transcript_expression_dict=None,
            transcript_expression_threshold=0.0,
            raise_on_variant_effect_error=True):
        """
        Parameters
        ----------
        variants : varcode.VariantCollection

        gene_expression_dict : dict
            Dictionary mapping gene IDs to RNA expression estimates

        gene_expression_threshold : float
            If gene_expression_dict is given, only keep effects on genes
            expressed above this threshold.

        transcript_expression_dict : dict
            Dictionary mapping transcript IDs to RNA expression estimates

        transcript_expression_threshold : float
            If transcript_expression_dict is given, only keep effects on
            transcripts above this threshold.

        raise_on_variant_effect_error : bool
        """
        # pre-filter variants by checking if any of the genes or
        # transcripts they overlap have sufficient expression.
        # I'm tolerating the redundancy of this code since it's much cheaper
        # to filter a variant *before* trying to predict its impact/effect
        # on the protein sequence.

        if gene_expression_dict:
            variants = _apply_filter(
                lambda variant: any(
                    gene_expression_dict.get(transcript_id, 0.0) >
                    gene_expression_threshold
                    for transcript_id in variant.gene_ids
                ),
                result_fn=variants.clone_with_new_elements,
                filter_name="Gene expression")
        if transcript_expression_dict:
            variants = _apply_filter(
                lambda variant: any(
                    transcript_expression_dict.get(transcript_id, 0.0) >
                    transcript_expression_threshold
                    for transcript_id in variant.transcript_ids
                ),
                variants,
                result_fn=variants.clone_with_new_elements,
                filter_name="Transcript expression")

        effects = variants.effects(
            raise_on_error=raise_on_variant_effect_error)

        return self.epitopes_from_mutation_effects(
            effects=effects,
            gene_expression_dict=gene_expression_dict,
            gene_expression_threshold=gene_expression_threshold,
            transcript_expression_dict=transcript_expression_dict,
            transcript_expression_threshold=transcript_expression_threshold)



# Copyright (c) 2015. Mount Sinai School of Medicine
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Common commandline arguments used by scripts
"""


from __future__ import print_function, division, absolute_import
import argparse
import logging

import mhctools
from mhctools.alleles import normalize_allele_name
import varcode

from .parsing_helpers import parse_int_list


arg_parser = argparse.ArgumentParser()

#
# Genomic Variants
#
variant_arg_group = arg_parser.add_argument_group(
    title="Variants",
    description="Genomic variant files")

variant_arg_group.add_argument("--vcf",
    default=[],
    action="append",
    help="Genomic variants in VCF format")

variant_arg_group.add_argument("--maf",
    default=[],
    action="append",
    help="Genomic variants in TCGA's MAF format",)

# TODO: add variant parsing from the commandline
#  variant_arg_group.add_argument("--variant",
#    default=[],
#    action="append",
#    help="Individual variant in a format such as chr1:3848C>G",)

def variant_collection_from_args(args):
    variant_collections = []
    for vcf_path in args.vcf:
        variant_collections.append(varcode.load_vcf(vcf_path))
    for maf_path in args.maf:
        variant_collections.append(varcode.load_maf(maf_path))

    if len(variant_collections) == 0:
        raise ValueError(
            "No variants loaded (use --maf or --vcf options)")
    elif len(variant_collections) == 1:
        return variant_collections[0]
    else:
        combined_variants = []
        for variant_collection in variant_collections:
            combined_variants.extend(list(variant_collection))
        return varcode.VariantCollection(combined_variants)

#
# Options passed to MHC binding predictor
#
mhc_options_arg_group = arg_parser.add_argument_group(
    title="MHC Prediction Options",
    description="MHC Binding Prediction Options")

mhc_options_arg_group.add_argument("--mhc-epitope-lengths",
    default=[8, 9, 10, 11],
    type=parse_int_list,
    help="Lengths of epitopes to consider for MHC binding prediction")

arg_parser.add_argument("--mhc-alleles-file",
    help="File with one HLA allele per line")

arg_parser.add_argument("--mhc-alleles",
    default="",
    help="Comma separated list of allele (default HLA-A*02:01)")

def mhc_alleles_from_args(args):
    alleles = [
        normalize_allele_name(allele.strip())
        for allele in args.mhc_alleles.split(",")
        if allele.strip()
    ]
    if args.mhc_alleles_file:
        with open(args.mhc_alleles_file, 'r') as f:
            for line in f:
                line = line.strip()
                if line:
                    alleles.append(normalize_allele_name(line))
    if len(alleles) == 0:
        raise ValueError(
            "MHC alleles required (use --mhc-alleles or --mhc-alleles-file)")
    return alleles

#
# MHC Binding Prediction
#

mhc_predictor_arg_group = arg_parser.add_mutually_exclusive_group(
    required=True)

mhc_predictor_flags = [
    ("--mhc-pan",
            "Use local NetMHCcons binding predictor",
            mhctools.NetMHCpan),
    ("--mhc-cons",
            "Use local NetMHCcons binding predictor",
            mhctools.NetMHCcons),
    ("--mhc-random",
        "Random values instead for MHC binding prediction",
        mhctools.RandomBindingPredictor),
    ("--mhc-smm", "Use local SMM binding predictor", None),
    ("--mhc-smm-pmbec", "Use local SMM-PMBEC binding predictor", None),
    ("--mhc-pan-iedb",
            "Use NetMHCpan via IEDB's web API",
            mhctools.IedbNetMHCpan),
    ("--mhc-cons-iedb",
            "Use NetMHCpan via IEDB's web API",
            mhctools.IedbNetMHCcons),
    ("--mhc-smm-iedb",
            "Use SMM via IEDB's web API",
            mhctools.IedbSMM),
    ("--mhc-smm-pmbec-iedb",
            "Use SMM-PMBEC via IEDB's web API",
            mhctools.IedbSMM_PMBEC),
]

for flag, help_string, mhc_class in mhc_predictor_flags:
    mhc_predictor_arg_group.add_argument(flag,
        default=False,
        action="store_true",
        help=help_string)

def mhc_binding_predictor_from_args(args):
    mhc_class = None
    for (flag, _, curr_mhc_class) in mhc_predictor_flags:
        arg_name = flag[2:].replace("-", "_")
        if getattr(args, arg_name):
            mhc_class = curr_mhc_class
            break
    if mhc_class is None:
        raise ValueError("No MHC prediction method specified")
    alleles = mhc_alleles_from_args(args)
    epitope_lengths = args.mhc_epitope_lengths
    logging.info(
        ("Building MHC binding prediction %s"
         " for alleles %s"
         " and epitope lengths %s") % (
            mhc_class.__class__.__name__,
            alleles,
            epitope_lengths))
    return mhc_class(
        alleles=alleles,
        epitope_lengths=epitope_lengths)
#
# Mutated sequence options
#
arg_parser.add_argument(
    "--padding-around-mutation",
    default=0,
    help="How many extra amino acids to include on either side of a mutation",
    type=int)

arg_parser.add_argument(
    "--self-filter-directory",
    help="Directory with 'self' ligand peptide sets, in files named by allele")


###
# RNA-Seq data
###

rna_group = arg_parser.add_argument_group(
    title="RNA-Seq",
    description="Transcript and gene abundance quantification")

rna_group.add_argument(
    "--rna-gene-fpkm-file",
    help="Cufflinks tracking file (FPKM measurements for Ensembl genes)",
    required=False)

rna_group.add_argument(
    "--rna-min-gene-expression",
    help="Minimum FPKM for gene expression",
    default=0.0,
    type=float)

rna_group.add_argument(
    "--rna-remap-novel-genes-onto-ensembl",
    help=(
        "If a novel gene is fully contained by known Ensembl gene, then "
        "merge their expression values"),
    default=False,
    action="store_true")

rna_group.add_argument(
    "--rna-transcript-fpkm-file",
    help="Cufflinks tracking file (FPKM measurements for Ensembl transcripts)")

rna_group.add_argument(
    "--rna-min-transcript-expression",
    help="Minimum FPKM for transcript expression",
    default=0.0,
    type=float)

#
# Filtering of epitopes
#
filter_group = arg_parser.add_argument_group(
    title="Filtering Options",
    description="Criteria for removing epitopes from results")

filter_group.add_argument(
    "--ic50-cutoff",
    help="Drop epitopes with predicted IC50 nM affinity above this value",
    default=None,
    type=float)


filter_group.add_argument(
    "--percentile-cutoff",
    help="Drop epitopes with predicted IC50 percentile rank above this value",
    default=None,
    type=float)

filter_group.add_argument(
    "--keep-wildtype-epitopes",
    help="Keep epitopes which do not contain mutated residues",
    default=False,
    action="store_true")

#
# Misc
#
arg_parser.add_argument(
    "--skip-variant-errors",
    default=False,
    action="store_true",
    help="Skip variants which cause errors")

#
# Output
#

output_group = arg_parser.add_argument_group(
    title="Output",
    description="Write results to different formats")

output_group.add_argument(
    "--output-csv",
    default=None,
    help="Path to output CSV file")

output_group.add_argument(
    "--output-html",
    default=None,
    help="Path to output HTML file")



# Copyright (c) 2015. Mount Sinai School of Medicine
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function, division, absolute_import

def parse_int_list(string):
    """
    Parses a string of numbers and ranges into a list of integers. Ranges
    are separated by "-" and inclusive of both the start and end number.

    Example:
        parse_int_list("8,10-12") == [8,10,11,12]
    """
    integers = []
    for substring in string.split(","):
        if "-" in substring:
            left, right = string.split("-")
            left_val = int(left.strip())
            right_val = int(right.strip())
            integers.extend(range(left_val, right_val + 1))
        else:
            integers.append(int(substring.strip()))
    return integers

def str2bool(value):
    return value.lower() in ('yes', 'true', 't', '1')



from mhctools import NetMHCpan
from nose.tools import eq_
from pyensembl import ensembl_grch37 as ensembl
from topiary import MutantEpitopePredictor
from varcode import Variant, VariantCollection

def _variant_collection():
    return VariantCollection([
        Variant(
          contig=10,
          start=100018900,
          ref='C',
          alt='T',
          ensembl=ensembl),
        Variant(
          contig=11,
          start=32861682,
          ref='G',
          alt='A',
          ensembl=ensembl)])

def _predictor(padding):
    return MutantEpitopePredictor(
      mhc_model_class=NetMHCpan,
      epitope_lengths=[9],
      padding_around_mutation=padding)

def _alleles():
    return [
      'A02:01',
      'a0204',
      'B*07:02',
      'HLA-B14:02',
      'HLA-C*07:02',
      'hla-c07:01'
    ]

def test_prediction_output():
    variant_collection = _variant_collection()
    predictor_without_padding = _predictor(padding=0)
    hla_alleles = _alleles()
    output_without_padding = predictor_without_padding.predict(
      variant_collection=variant_collection,
      mhc_alleles=hla_alleles)
    # one prediction for each variant * number of alleles
    eq_(len(output_without_padding.strong_binders(500.0)), 12)
    predictor_with_padding = _predictor(padding=0)
    output_with_padding = predictor_with_padding.predict(
      variant_collection=variant_collection,
      mhc_alleles=hla_alleles)
    eq_(len(output_with_padding), 636)
    df = output_with_padding.dataframe()
    binders = df[df['MHC_IC50'] < 500]
    eq_(len(binders), 12)



# Copyright (c) 2014. Mount Sinai School of Medicine
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import os

from setuptools import setup

readme_dir = os.path.dirname(__file__)
readme_filename = os.path.join(readme_dir, 'README.md')

try:
    with open(readme_filename, 'r') as f:
        readme = f.read()
except:
    readme = ""

try:
    import pypandoc
    readme = pypandoc.convert(readme, to='rst', format='md')
except:
    print(
        "Conversion of long_description from MD to reStructuredText failed...")


if __name__ == '__main__':
    setup(
        name='topiary',
        version="0.0.5",
        description="Predict cancer epitopes from cancer sequence data",
        author="Alex Rubinsteyn, Tavi Nathanson",
        author_email="alex {dot} rubinsteyn {at} gmail {dot} com",
        url="https://github.com/hammerlab/topiary",
        license="http://www.apache.org/licenses/LICENSE-2.0.html",
        classifiers=[
            'Development Status :: 3 - Alpha',
            'Environment :: Console',
            'Operating System :: OS Independent',
            'Intended Audience :: Science/Research',
            'License :: OSI Approved :: Apache Software License',
            'Programming Language :: Python',
            'Topic :: Scientific/Engineering :: Bio-Informatics',
        ],
        install_requires=[
            'numpy >=1.7, <2.0',
            'pandas >=0.13.1, <0.17.0',
            'mhctools >=0.0.0, <0.1.0',
            'varcode >=0.3.1, <0.4.0',
            'nose >=1.3.6, <1.4'
        ],
        long_description=readme,
        packages=['topiary'],
        scripts=[
            'scripts/topiary'
        ],
    )
